# get_zfs-lib.pl
#
# (c) napp-it 2010-2024
# fill %zfs with 'zfs get all' and editable parameter with 'zfs set'
# basics from zfs list in %zpool, already set in getbasics-lib.pl (allpools, syspool, datapools)

# jan 2024 Win

use strict;
use Fcntl qw(:flock);
our (%zfs, %sys, %cfg, %current);

#&print_hash(%zfs);
# zfs and zpool properties
if ($current{'on'}=~/Oracle Solaris/) {
   &get_solaris;                     # add openzfs prop
} else {
   &get_openzfs;                     # add openzfs prop
}


################
sub get_solaris {
#################
   my ($r,@p,@l,@z,$s,$k);

   my @pools=split(/\t/,$zfs{'allpools'});
   foreach my $p (@pools)  {
      $r=&exe("zpool get -H all $p");
      @l=split(/\n/,$r);

      # &print_array(@l);

      # [0] -> 'NAME   PROPERTY       VALUE                     SOURCE'
      # [1] -> 'rpool  allocated      11.4G                     -'
      # [2] -> 'rpool  altroot        -                         defaul

      foreach my $l (@l) {
             @p=split(/\t/,$l);

             #&print_array(@p);

             # [0] -> 'rpool'
             # [1] -> 'allocated'
             # [2] -> '11.4G'
             # [3] -> '-'

             $t=$p."_$p[1]";
             $zfs{$t}=$p[2];
      }

   }

   #all datasets
   $zfs{'allds'}=&exe("zfs list -H -o name");
   $zfs{'allds'}=~s/\n/\t/gs;

   # all user filesystems
   $zfs{'allfs'}="";

   @t=split(/\t/,$zfs{'allds'});
   foreach my $t (@t) {                                                       # open-zfs see get_openzfs

        # Oracle Solaris
        if ($current{'on'} =~/solaris/i)  {
          if ($t=~/^rpool\/ROOT/)       { next; }             # Solaris
          if ($t=~/^rpool\/VARSHARE/)   { next; }             # Solaris
          if ($t=~/^rpool\/dump|^rpool\/export|^rpool\/swap/)  { next; }          # Solaris
        }

        $zfs{'allfs'}.="$t\t";
   }
   $zfs{'allfs'}=~s/\t$//;

   #
   #zfs properties
   $r=&exe("zfs get -H all");
   @l=split(/\n/,$r);

   # &print_array(@l);

   # [0] -> 'NAME   PROPERTY       VALUE                     SOURCE'
   # [1] -> 'rpool  allocated      11.4G                     -'
   # [2] -> 'rpool  altroot        -                         defaul

   foreach my $l (@l) {
             @p=split(/\t/,$l);

             #&print_array(@p);

             # [0] -> 'rpool'
             # [1] -> 'allocated'
             # [2] -> '11.4G'
             # [3] -> '-'

             $t=$p[0]."_$p[1]";
             $zfs{$t}=$p[2];

             # set default off, set later
             $t=$p[0]."_sharesmb";
             $zfs{$t}="off";

   }  #/ foreachlines  properties

   #&print_hash(%zfs);
}



##############
sub get_openzfs  {                #basic info all zfs
##############
  #my (@w,$w,$w,@l,$l,$zfs,$option,$ant,$p,$name,$prot,$path, $iro, $irw, $inone, $f,$r,@data,%pos);
   my (@w,$w,$w,@l,$l,$t,$zfs,$p_zfs,$p_zfsmin,$p_zpool,$p_zpoolmin,$option,$ant,$p,@p,$name,$prot,$path,$iro,$irw,$inone,$f,$r,@data,%pos,$zpooldata,$zfsdata);


  #########################
  #all zfs prop for all zfs
  #########################

  #zfs properties
  # from 'zfs list -o all', load only for single filesystems
  if ($current{'on'}=~/win32/i) {
     $p_zfs='name,creation,used,avail,refer,ratio,mounted,origin,quota,reservation,volsize,volblock,recordsize,mountpoint,sharenfs,checksum,compression,atime,devices,exec,setuid,rdonly,zoned,snapdir,aclmode,aclinherit,createtxg,canmount,xattr,copies,version,utf8only,normalization,case,vscan,nbmand,sharesmb,refquota,refreserv,guid,primarycache,secondarycache,usedsnap,usedds,usedchild,usedrefreserv,defer_destroy,userrefs,logbias,objsetid,dedup,mlslabel,sync,dnsize,refratio,written,clones,lused,lrefer,fslimit,sslimit,fscount,sscount,snapdev,acltype,context,fscontext,defcontext,rootcontext,relatime,redund_md,overlay,resumetok,encryption,keylocation,keyformat,pbkdf2iters,encroot,keystatus,special_small_blocks,rsnaps,snapshots_changed,prefetch,zvol,threading,com.apple.mimic_hfs,driveletter';           # all zfs properties

  } else {  # open-zfs without driveletter
     $p_zfs='name,creation,used,avail,refer,ratio,mounted,origin,quota,reservation,volsize,volblock,recordsize,mountpoint,sharenfs,checksum,compression,atime,devices,exec,setuid,rdonly,zoned,snapdir,aclmode,aclinherit,createtxg,canmount,xattr,copies,version,utf8only,normalization,case,vscan,nbmand,sharesmb,refquota,refreserv,guid,primarycache,secondarycache,usedsnap,usedds,usedchild,usedrefreserv,defer_destroy,userrefs,logbias,objsetid,dedup,mlslabel,sync,dnsize,refratio,written,clones,lused,lrefer,fslimit,sslimit,fscount,sscount,snapdev,acltype,context,fscontext,defcontext,rootcontext,relatime,redund_md,overlay,resumetok,encryption,keylocation,keyformat,pbkdf2iters,encroot,keystatus,special_small_blocks,rsnaps,snapshots_changed,prefetch,zvol,threading';           # all open-zfs zfs properties
  }
  #always=min loaded
  if ($current{'on'}=~/win32/i) {
     $p_zfsmin='name,creation,used,avail,refer,ratio,mounted,origin,quota,reservation,recordsize,mountpoint,compression,atime,rdonly,snapdir,aclmode,aclinherit,canmount,version,utf8only,normalization,case,nbmand,refquota,refreserv,guid,primarycache,secondarycache,dedup,sync,clones,encryption,keylocation,keyformat,pbkdf2iters,encroot,keystatus,special_small_blocks,driveletter';  # min zfs properties
  } else {  # open-zfs without driveletter
     $p_zfsmin='sharesmb,sharenfs,name,creation,used,avail,refer,ratio,mounted,origin,quota,reservation,recordsize,mountpoint,compression,atime,rdonly,snapdir,aclmode,aclinherit,canmount,version,utf8only,normalization,case,nbmand,refquota,refreserv,guid,primarycache,secondarycache,dedup,sync,clones,encryption,keylocation,keyformat,pbkdf2iters,encroot,keystatus,special_small_blocks';  # min zfs properties
  }

  #debug
  #   $p_zfsmin='name,type,creation,used,avail,refer,ratio,mounted,origin,quota,reservation,recordsize,mountpoint,compression,atime,rdonly,snapdir,aclmode,aclinherit,canmount,version,utf8only,normalization,case,nbmand,refquota,refreserv,guid,primarycache,secondarycache,dedup,sync,clones,encryption,keylocation,keyformat,encroot,keystatus,special_small_blocks';  # min zfs properties


  # zpool properties
  # from 'zpool list -o all' load only for single filesystems
  $p_zpool='name,size,cap,altroot,health,guid,version,bootfs,delegation,autoreplace,cachefile,failmode,listsnaps,autoexpand,dedup,free,allocated,rdonly,ashift,comment,expandsz,freeing,frag,leaked,multihost,checkpoint,load_guid,autotrim,compatibility,bclone_used,bclone_saved,bclone_ratio,feature@async_destroy,feature@empty_bpobj,feature@lz4_compress,feature@multi_vdev_crash_dump,feature@spacemap_histogram,feature@enabled_txg,feature@hole_birth,feature@extensible_dataset,feature@embedded_data,feature@bookmarks,feature@filesystem_limits,feature@large_blocks,feature@large_dnode,feature@sha512,feature@skein,feature@edonr,feature@userobj_accounting,feature@encryption,feature@project_quota,feature@device_removal,feature@obsolete_counts,feature@zpool_checkpoint,feature@spacemap_v2,feature@allocation_classes,feature@resilver_defer,feature@bookmark_v2,feature@redaction_bookmarks,feature@redacted_datasets,feature@bookmark_written,feature@log_spacemap,feature@livelist,feature@device_rebuild,feature@zstd_compress,feature@draid,feature@zilsaxattr,feature@head_errlog,feature@blake3,feature@block_cloning,feature@vdev_zaps_v2,feature@redaction_list_spill,feature@raidz_expansion'; # all zpool properties

  # always=min loaded
  $p_zpoolmin='name,size,cap,altroot,health,guid,version,autoreplace,failmode,autoexpand,dedup,free,allocated,frag,checkpoint,autotrim,readonly';

  #all datasets
  $zfs{'allds'}=&exe("zfs list -H -o name");
  $zfs{'allds'}=~s/\n/\t/gs;

  # step 1, get zfs properties  for $current{'member_ip'}
  if ($zfs{'allpools'}=~/\w/) {
       if ($in{'prop'} ne "zfsdetails")  {

           # opt read cache via socket
           $ant=&exe("zfs get -H -t filesystem $p_zfsmin");   # min zfs properties of all pools, zfs get readonly -> zfs property                                                                                         # if loaded as file, rename

           $zfs{'zfs_properties'}="$p_zfsmin";

       } else { # all

           $ant=&exe("zfs get -H -t filesystem $p_zfs");          # all zfs properties of all pools
           $zfs{'zfs_properties'}="$p_zfs";
       }

#&mess($ant);

       $zfsdata=$ant;
       # tank2     ->driveletter     ->-         ->default
       # tank2/xx  ->name            ->tank2/xx  ->-

       #@t=split(/\n/,$ant); &print_array(@t);
       #&mess($ant);

       @l=split(/\n/,$ant);
       foreach $l (@l) {
              @p=split(/\t/,$l);

              #[0] -> 'drei'          fs
              #[1] -> 'name'          prop
              #[2] -> 'drei'          value
              #&print_array(@p);

              $f="$p[0]\_$p[1]";        # tank_name
              $zfs{$f}=$p[2];           # t->  tank
       }

       #&print_hash(%zfs);
       #@t=split(/\t/,$ant); @t=sort @t; &print_array(@t);

       ###############
       #get zpool properties
       ###############

             # step 2, get zpool properties

            if ($in{'prop'} ne "pooldetails")  {

                # read via socket, opt cache
                $ant=&exe("zpool get -H $p_zpoolmin");   # min zfs properties of all pools, rename upload
                                                                                                # if upload: opt2 is filename

                $zfs{'zpool_properties'}="$p_zpoolmin";

            } else { # all

                $ant=&exe("zpool get -H $p_zpool");      # all zfs properties of all pools,#  zpool readonly -> zpool property -> poolreadonly
                $zfs{'zpool_properties'}="$p_zpool";
            }

  #&mess($ant);
            $zpooldata=$ant;
            # tank2     ->driveletter     ->-         ->default
            # tank2/xx  ->name            ->tank2/xx  ->-

            #@t=split(/\n/,$ant); &print_array(@t);
            #&mess($ant);

            @l=split(/\n/,$ant);
            foreach $l (@l) {
                   @p=split(/\t/,$l);

                   #[0] -> 'drei'          fs
                   #[1] -> 'name'          prop
                   #[2] -> 'drei'          value
                   #&print_array(@p);

                   if ($p[1] eq "readonly") { $p[1]="poolreadonly"; }

                   $f="$p[0]\_$p[1]";        # tank_name
                   $zfs{$f}=$p[2];           # t->  tank
            }

            #&print_hash(%zfs);
            #@t=split(/\t/,$ant); @t=sort @t; &print_array(@t);

  } # if pools?


   # all user filesystems
   $zfs{'allfs'}="";

   @t=split(/\t/,$zfs{'allds'});
   foreach my $t (@t) {

        #bsd
        if ($current{'on'} =~/bsd/i)   {
           if ($t=~/^zroot\//)           { next; }       # free-bsd
           if ($t=~/^boot-pool\//)       { next; }       # truenas core?
        }

        #linux
        if ($current{'on'} =~/linux/i)    {
          if ($t=~/^boot-pool\//)         { next; }        # truenas scale
        }

        # Illumos
        if ($current{'on'} =~/solaris/i)   {       # illumos
          if ($t=~/^rpool\/ROOT/)          { next; }      # BE
          if ($t=~/^rpool\/dump|^rpool\/swap/)   { next; }      # dump and swap
          if ($t=~/^rpool\/export/)        { next; }      # home?
          if ($t=~/^zones\//)              { next; }      # smartos

                                                   # see get_solaris for Oracle Solaris
        }
        $zfs{'allfs'}.="$t\t";
   }
   $zfs{'allfs'}=~s/\t$//;

  #&mess($zpooldata);
  #&print_hash(%zfs);




######################################################################
#####################################################################
if (0) {        #skip  ?
     ################################################################
    # get all fs and main properties
    # on S11: sharenfs and sharesmb gives "- off" wheras OI gives "off"

    @l=();
    @l=split(/\n/,$zfsdata);
    @l=sort(@l);

  #debug
  #&print_array(@l);
  #&print_hash(%zfs);
  #&print_hash(%pos);
  #&mess($sys{'auto_os'});

    $zfs{'allfs'}="";
    $zfs{'allvol'}="";

    my %ls_todo;

    foreach my $l (@l) {

          #if ($l=~/^NAME +TYPE/) { next; }

          #$l=~s/  +/\t/g;
          @w=();
          @w=split(/\t/,$l);
   #&print_array(@w);

          #if ($w[$pos{'type'}] eq "filesystem") {
             $zfs{'allfs'}.="$w[0]\t";   #name des fs
          #}
          if ($w[$pos{'type'}] eq "volume") {
             $zfs{'allvol'}.="$w[0]\t";  #name des volume
          }


          $w="$w[0]_sharesmb";                       #chris
          $zfs{$w}=$w[$pos{'sharesmb'}];
          $w="$w[0]_sharenfs";
          $zfs{$w}=$w[$pos{'sharenfs'}];

          $w="$w[0]_origin";
          $zfs{$w}=$w[$pos{'origin'}];

          # add to zfs{'clones'}
          if ($w[$pos{'origin'}]=~/\@/) {
            $zfs{'all_clones'}.="$w[0]->$w[$pos{'origin'}]\t";
          }

          $w="$w[0]_recsize";
          $zfs{$w}=$w[$pos{'recsize'}];

          $w="$w[0]_avail";
          $zfs{$w}=$w[$pos{'avail'}];

          $w="$w[0]_used";
          $zfs{$w}=$w[$pos{'used'}];

          $w="$w[0]_readonly";
          $zfs{$w}=$w[$pos{'rdonly'}];

          $w="$w[0]_reservation";
          $zfs{$w}=$w[$pos{'reserv'}];

          $w="$w[0]_refreservation";
          $zfs{$w}=$w[$pos{'refreserv'}];

          $w="$w[0]_refquota";
          $zfs{$w}=$w[$pos{'refquota'}];

          $w="$w[0]_quota";
          $zfs{$w}=$w[$pos{'quota'}];

          $w="$w[0]_mountpoint";

          #
          #$zfs{$w}=$w[$pos{'mountpoint'}];    #chris
          $t=$w[$pos{'mountpoint'}];
          $zfs{$w}=$t;
          # As an optimization, forego an `ls` for certain cases where irrelevant.
          if ($t eq "-" || $t eq "legacy" || $t eq "/" || $t eq "/a") {
              $w="$w[0]_ls";
              $zfs{$w}="-";

              $w="$w[0]_dv";
              $zfs{$w}=" "; # one space for _dv

              $w="$w[0]_shdv";
              $zfs{$w}=""; # empty for _shdv
          } else {
              $ls_todo{$w[0]} = $t;
          }
          #/

           $w="$w[0]_compression";
          $zfs{$w}=$w[$pos{'compress'}];

          $w="$w[0]_nbmand";
          $zfs{$w}=$w[$pos{'nbmand'}];

          $w="$w[0]_dedup";
          $zfs{$w}=$w[$pos{'dedup'}];

          $w="$w[0]_sync";
          $zfs{$w}=$w[$pos{'sync'}];
    } #/ foreach, chris

    #  chris
    #  Determine an OS-specific command for getting ZFS encryption options.
    my $zfs_getenc = ($sys{'auto_os_type'} =~ /\b(sol_)/) ?
      "zfs get -H encryption,keystatus,enc_split:,keysource" :
      # ill_*; OpenZFS such as Linux, Darwin; etc.
      "zfs get -t filesystem -H encryption,keystatus,enc_split:,encryptionroot";

    $t=&exe($zfs_getenc,'','get zfs' . __LINE__);
    chomp($t);
    if (!$?) {
       @l=split(/\n/,$t);
       foreach my $l (@l) {
          @w=split(/\t/,$l);       # zfs  prop  value  source
          if (exists $zfs{"$w[0]_origin"}) {
             if ($w[1] eq "keysource") {
                ### martingo 20180306 >>> Der Wert SOURCE (local/default/-/inherited...) wird unter neu angelegtem Schluessel angelegt
                # LIMIT 4 wegen Leerzeichen in inherited from ...
                @w=split(/\t/,$l,4);       # zfs  prop  value  source
                $w="$w[0]_keysource_inherited_from";
                ($zfs{$w}) = $w[3] =~ /inherited from (.+)/;
             } else {
                $w="$w[0]_$w[1]";
                $zfs{$w}=$w[2];
             }
          }
       }
    }

   # Try to determine an OS-specific command for `ls -l` with a fixed time-style.
   my ($ls_fixedtime, $fixedtime_len);
   if ($sys{'auto_os'} eq "Linux") {
     $ls_fixedtime = "$sys{'binpath'}ls -dl --time-style=iso";
     # e.g.
     # drwxr-xr-x. 11 root root 8192 07-25 03:20 /var/log
     # e.g.
     # drwxr-xr-x. 11 root root 8192 2002-03-30  /var/oldlog
     $fixedtime_len = 12;
   } elsif ($sys{'auto_os_type'} =~ /\b(ill_|sol_)/) {
     $ls_fixedtime = "$sys{'binpath'}ls -de";
     # e.g.
     # drwxr-xr-x   7 root     sys            8 Sep  5 15:10:01 2020 opt
     $fixedtime_len = 21;
   } elsif ($sys{'auto_os_type'} eq "dar") {
     $ls_fixedtime = "$sys{'binpath'}ls -dlT";
     # e.g.
     # drwxrwxr-x  69 root  admin  2208 Jul 23 22:31:41 2021 Applications
     $fixedtime_len = 21;
   }

   # Get the `ls -l` in large batches.
   if (! defined $ls_fixedtime) {
  #   warn("No known fixed time-style for `ls` under $sys{'auto_os'}");
   } else {
     my $s3_dmsys_rel = "S3_data/.minio.sys";
     my %mountpoints_todo = map { ($_, undef) } values %ls_todo; # get unique
     my @mountpoints_todo = keys %mountpoints_todo;
     # Also test for an S3_data indicator directory for each mountpoint.
     @mountpoints_todo = map { ($_, "$_/$s3_dmsys_rel")} @mountpoints_todo;
     while (@mountpoints_todo > 0) {
       my @mountpoints_batch = map { quotemeta($_) } splice @mountpoints_todo, 0, 200;
       $t=&exe("$ls_fixedtime /dev/null @mountpoints_batch", "n");
       chomp($t);
       @l=split(/\n/,$t);
       foreach my $l (@l) {
          if ($l =~ /^.(?:[-r][-w][-x]){3}/x) {
            my @ll = split /\s+/, $l, 6;
            my $lsname = substr $ll[5], $fixedtime_len;
            # Strip if $lsname has e.g. /bin -> ./usr/bin
            $lsname =~ s/\s+ -> .+//x;
            $mountpoints_todo{$lsname} = $l;
          }
       }
     }
     while (my ($dataset, $mountpoint) = each %ls_todo) {
       $w="${dataset}_ls";
       $zfs{$w} = (exists $mountpoints_todo{$mountpoint} &&
         defined $mountpoints_todo{$mountpoint}) ?
         $mountpoints_todo{$mountpoint} : "-";

       $w="${dataset}_hasS3";
       $zfs{$w} = exists $mountpoints_todo{"$mountpoint/$s3_dmsys_rel"};
     }
   }

   # Get a bulk list of any shares, and then get ACLs for the discovered shares.
   {
     my $zfs_shares_rel = ".zfs/shares/";
     my %sharelist_todo = map { ("$_/$zfs_shares_rel", undef) } values %ls_todo; # get unique
     my @sharelist_todo = keys %sharelist_todo;
     my %acls_todo;

     while (@sharelist_todo > 0) {
       my @sharelist_batch = map { quotemeta($_) } splice @sharelist_todo, 0, 200;
       $t=&exe("$sys{'binpath'}ls /dev/null @sharelist_batch", "n");
       # e.g.
       # /dev/null
       #
       # /tank/fs1/.zfs/shares/:
       # share1
       #
       # /dozer/fs2/.zfs/shares/:
       # share2
       while ($t =~ m`(/[^\n]+?):\n (\S[^\n]*)\n`gsx) {
         my $share = "$1$2";
         $sharelist_todo{$1} = $share;
         $acls_todo{$share} = undef;
       }
     }

     my @acls_todo = keys %acls_todo;
     while (@acls_todo > 0) {
       my @acls_batch = map { quotemeta($_) } splice @acls_todo, 0, 200;
       $t=&exe("$sys{'binpath'}ls -dV @acls_batch", "n");
       # e.g.
       # -rwxrwxrwx+  1 root     root           0 Jul 18 10:43 /tank/fs1/.zfs/shares/share1
       #               everyone@:rwxpdDaARWcCos:-------:allow
       # -rwxrwxrwx+  1 root     root           0 Jul 18 10:43 /dozer/fs2/.zfs/shares/share2
       #               everyone@:rwxpdDaARWcCos:-------:allow
       my @s = split /(?<=\n)(?=\S)/x, $t;
       for (my $i = 0; $i < @s; ++$i) {
         my $acl = $s[$i];
         if ($acl =~ m`(/[^\n]+)`x) {
           $acls_todo{$1} = $acl;
         }
       }
     }

     while (my ($dataset, $mountpoint) = each %ls_todo) {
       $w="${dataset}_shdv";
       # N.b. use empty string as fallback for _shdv.
       my $acl = "";
       my $mpzsr = "$mountpoint/$zfs_shares_rel";
       if (exists $sharelist_todo{$mpzsr} && defined $sharelist_todo{$mpzsr}) {
         my $share = $sharelist_todo{$mpzsr};
         if (exists $acls_todo{$share} && defined $acls_todo{$share}) {
           $acl = $acls_todo{$share};
         }
       }
       $zfs{$w} = $acl;
     }
   }

   # Get a bulk list of ACLs for the mountpoints.
   {
     my %acls_todo = map { ($_, undef) } values %ls_todo; # get unique
     my @acls_todo = keys %acls_todo;
     while (@acls_todo > 0) {
       my @acls_batch = map { quotemeta($_) } splice @acls_todo, 0, 200;
       $t=&exe("$sys{'binpath'}ls -dV @acls_batch", "n");
       # e.g.
       # drwxr-xr-x   7 root     sys            8 Sep  5  2020 /opt/
       #                  owner@:rwxp--aARWcCos:-------:allow
       #                  group@:r-x---a-R-c--s:-------:allow
       #               everyone@:r-x---a-R-c--s:-------:allow
       # drwxr-xr-x  46 root     sys           46 May 19  2018 /var/
       #                  owner@:rwxp--aARWcCos:-------:allow
       #                  group@:r-x---a-R-c--s:-------:allow
       #               everyone@:r-x---a-R-c--s:-------:allow
       my @s = split /(?<=\n)(?=\S)/x, $t;
       for (my $i = 0; $i < @s; ++$i) {
         my $acl = $s[$i];
         if ($acl =~ m`(/[^\n]+)`x) {
           $acls_todo{$1} = $acl;
         }
       }
     }
     while (my ($dataset, $mountpoint) = each %ls_todo) {
       $w="${dataset}_dv";
       # N.b. use one space as a fallback for _dv.
       $zfs{$w} = (exists $acls_todo{$mountpoint} &&
         defined $acls_todo{$mountpoint}) ? $acls_todo{$mountpoint} : " ";
     }
   }

  #&print_hash(%zfs);

   $zfs{'allvol'}=~s/\t$//;
   $zfs{'allfs'}=~s/\t$//;
   $zfs{'all_clones'}=~s/\t$//;


   #add all properties for preselected zfs
   $zfs=$zfs{'add_details'};
   if ($zfs=~/ /) { $zfs=""; }
   if ($zfs ne "") {
     $ant=&exe("$sys{'sbinpath'}zfs get -H all $zfs",'','get zfs 166');
     if ($ant=~/not exist/s) { $ant=""; }
   }



  ## stage 1b Solaris 11
  ######################
  # set sharesmb and sharenfs are obsolet, use new zfs set share=name=xx,path=xx,prot=nfs/smb zfs
  # get shareinfo smb+nfs: zfs get -H -o name,value share
  # ex
  # tank/data     name=zyz,path=/tank/data,prot=smb        rw=, ro=, none=, guestok=true
  # tank/data     name=zyy,path=/tank/data,prot=nfs        rw=, ro=, none=

  if ($sys{'auto_os'} eq "Solaris 11") {
       $ant=&exe("zfs get -H -o name,value share",'','get zfs 181');
       @l=();
       @l=split(/\n/,$ant);
       foreach $l (@l) {

                $prot="smb";
                if ($l=~/prot=nfs/) { $prot="nfs"; }

                $zfs=$l; $zfs=~s/\s+name=.*//;
                $l=~s/^.*name=/name=/;
                @w=();
                @w=split(/,/,$l);
                $iro="1";
                $irw="1";
                $inone="1";
                foreach $w (@w) {                   # commavalues
                         if ($w=~/name=/) {          # name=abc
                              $w=~s/name=//;
                              $t="$zfs\_$prot"."name";
                              $zfs{$t}.="$w\t";          #tank/data_smbname -> abc , multiple namse possible
                         }
                         if ($w=~/path=/) {          # name=abc
                              $w=~s/path=//;
                              $t="$zfs\_$prot"."path";
                              $zfs{$t}=$w;          #
                         }
                         if ($w=~/ro=/) {            # ro=..
                              $w=~s/ro=//;
                              $t="$zfs\_$prot"."ro$iro";
                              $zfs{$t}=$w;          #
                              $iro++;
                         }
                         if ($w=~/rw=/) {            # rw=..
                              $w=~s/rw=//;
                              $t="$zfs\_$prot"."rw$irw";
                              $zfs{$t}=$w;          #
                              $irw++;
                         }
                         if ($w=~/none=/) {            # none=..
                              $w=~s/none=//;
                              $t="$zfs\_$prot"."none$inone";
                              $zfs{$t}=$w;          #
                              $inone++;
                         }
                         if ($w=~/guestok=/) {         # guestok=..
                              $w=~s/guestok=//;
                              $t="$zfs\_$prot"."guestok";
                              $zfs{$t}=$w;          #
                         }
                }
       }
  }

  # opt needed
          my @prop;
          my @edit;
          my @inherit;
          my @values;

  ####################################################
  #stage2 get editable properties if details are asked
  ####################################################

  #test
  #$zfs{'add_details'}="1";

    if ($zfs{'add_details'} eq "1") {

            @w=split(/\t/,$zfs{'properties'});
            @l=();


            foreach $w (@w)  {
                  $w=~s/^\s+//;                                #empty line
                  if ($w eq "") { next; }
                  if ($w =~/^PROPERTY/) { next; }        #PROPERTY  EDIT  INHERIT  VALUES

                  $w=~s/  +/\t/;                        #property \t edit
                  $w=~s/  +/\t/;                        #property \t edit  \t inherit
                  $w=~s/  +/\t/;                        #property \t edit  \t inherit  \t values

                  @l=();
                  @l=split(/\t/,$w);

                  if (!($zfs{'properties'}=~/\b$l[0]\b/)) {
                     $zfs{'properies'}.="$l[0]\t";        #list of properties
                  }

                  $t="editable_$l[0]";
                  $zfs{$t}=$l[1];

                  $t="inherit_$l[0]";
                  $zfs{$t}=$l[2];

                  $t="values_$l[0]";
                  $zfs{$t}=$l[3];

                  #push (@l,$w);
          }
          $zfs{'properies'}=~s/\t$//;
    }


  ##################################
  #stage3 get volumes and info about volumes (iscsi as a zfs property)
  ##################################



                    $ant=&exe("zfs list -H -t volume",'','get zfs 291');                #list of all volumes
                                  # NAME              USED  AVAIL  REFER  MOUNTPOINT                # -H
                                  # data/comstar/xx  1.50G   731G    16K  -
                                  # syspool/swap     1.03G  2.23G    16K  -

  #&mess($ant);
                    @w=(); @w=split(/\n/,$ant);
                    foreach $w (@w)  {
                          $w=~s/ +/\t/g;
                          @l=();
                          @l=split(/\t/,$w);

                          $zfs{'volumes'}.="$l[0]\t";        #list of volumes

                          $t="$l[0]\_used";
                          $zfs{$t}=$l[1];

                          $t="$l[0]\_avail";
                          $zfs{$t}=$l[2];

                          $t="$l[0]\_refer";
                          $zfs{$t}=$l[3];

                          $t="$l[0]\_mountpoint";
                          $zfs{$t}=$l[4];

                          $t="$l[0]\_type";
                          $zfs{$t}='volume';

                          #iscsi_vol
                          if (!($w=~/\/iscsi_/)) { next; }

                          $t=$w; $t=~s/\/iscsi_.*//;              # zfs ex tank/data
                          $r=$w; $r=~s/.*iscsi_//;  $r=~s/\s.*//;  # id ex 1386116366
                          $t=$t."_iscsi_vol";
                          $zfs{$t}=$r;

                          # add size (avail+used)                         $current{'iscsi_my_size'}   =&zfslib_val2xb($v[1]/1000);            # in Bytes not KB
                          $p= &zfslib_val2kb($l[1]) + &zfslib_val2kb($l[2]);  # used + avail in KB

                          $t=$t."_size";
                          $zfs{$t}=$p;

                          $t.="2";                     # value in GB
                          $p=&zfslib_val2xb($p);
                          $zfs{$t}=$p;
                  }
                  $zfs{'volumes'}=~s/\t$//;


    # get default LU of a fs
                #  $ant=&exe("$sys{'sbinpath'}sbdadm list-lu | grep iscsi_",'',__LINE__);            # list of iscsi LU
                  chomp($ant);
                  @w=(); @w=split(/\n/,$ant);
  #&print_array(@w);
                  foreach $w (@w)  {
                      chomp($w);
                      $t=$w;
                      $t=~s/.*\/rdsk\///;
                      $t=~s/\/iscsi_.*//;      # keep zfs
                      $w=~s/\s.*//;            # LU
  #&mess($w);
                      if ($w=~/^\d+/) {
                         $t=$t.="_iscsi_LU";
                         $w=~tr/a-z/A-Z/;                                            # always uppercase
                         $zfs{$t}=$w;

                         # view?
                         $r=&exe("$sys{'sbinpath'}stmfadm list-view -l $w"); chomp($r);      # View Entry or no views
                         if ($r=~/View/s) {
                             $t.="_view";
                             $zfs{$t}=$r;
                            # &mess($r);
                         }

                      }
                  }





  #stage4 get snaps and info about

  #test
  #$zfs{'add_snaps'}="1";

          if ($zfs{'add_snaps'} eq "1") {
                    #
                    $ant="";
                    @data=();

                    # check accelerator
                    if ((-f "/tmp/nappit/zfs_last.zfs") && ($cfg{'ver'} ne "")) {
                            open (F,"</tmp/nappit/zfs_last.zfs");
                                  flock(F, LOCK_EX);
                                  @data=<F>;
                            close (F);
                            $t=time();

                            # read acc
                            if (($t-$data[0] < 1200) && (-f "/tmp/nappit/zfs_list_snaps.zfs"))   { # max 20min
                                 @data=();
                                 open (F,"</tmp/nappit/zfs_list_snaps.zfs");
                                    flock(F, LOCK_EX);
                                    @data=<F>;
                                 close (F);
                                 $ant=join("",@data);
                                 &exe("echo acc_read_zfs snaps");
                            } else {
                                 &exe("rm /tmp/nappit/*.zfs");
                            }

                            if (!($ant=~/\@/s)) { $ant=""; }
                    }

                    #$ant="";
                    if ($ant eq "") {

                                  # zfs list -r -t snapshot -o name,creation,used,refer
                                   $ant=&exe("zfs list -r -t snapshot -o name,creation,used,avail,refer -S creation",'','');
                                   $ant=&parse_date($ant);
                                   #alt $ant=&exe("zfs list -H -t snapshot",'','get zfs 330');                #list of all snapshots
                                             # NAME                                      USED    AVAIL  REFER  MOUNTPOINT                # -H
                                             # data@hourly_1273532400               202M      -   202M    -
                                             # data@hourly_1273539601                21K      -    27K    -
                    }

                    @w=split(/\n/,$ant);
                    foreach $w (@w)  {
                          if (!($w=~/\@/))  { next; }
                          $w=~s/ +/\t/g;
                          @l=();
                          @l=split(/\t/,$w);

                          if (!($l[0]=~/^$zfs{'syspool'}/)) {           #sys_snap
                                  $zfs{'snaps_data'}.="$l[0]\t";        #list of data snaps
                          } else {
                                  $zfs{'snaps_sys'}.="$l[0]\t";        #list of sys snaps
                          }

                          $t="$l[0]\_used";
                          $zfs{$t}=$l[2];

                          $t="$l[0]\_avail";
                          $zfs{$t}=$l[3];

                          $t="$l[0]\_refer";
                          $zfs{$t}=$l[4];


                          $t="$l[0]\_type";
                          $zfs{$t}='snapshot';

                  }
                  $zfs{'snaps_data'}=~s/\t$//;                       #data snaps
                  $zfs{'snaps_sys'}=~s/\t$//;                        #sys snaps
          }



  #stage 6
  #get afp shares
  ###############
          # netatalk 3 detection admin.pl 904
          if ($sys{'netatalk3'} ne "") {

                  $ant=&exe("cat $sys{'netatalk3'}",'','get zfs 376');

                  #get shares
                  # [share]
                  # path= /a/b

                  @l=split(/\[/,$ant);
                  foreach $l (@l) {
                                    if (!($l=~/^\w+\]\n/)) { next; }                  # not a header like: [share]
                                    if ($l=~/^Global]\n/)  { next; }                  # Global header [Global]
                                    if ($l=~/^Homes]\n/)   { next; }                  # Home header [Homes]

                                    @w=();
                                    @w=split(/\n/,$l);

                                    $zfs="";
                                    # get zfs (path)
                                    foreach $w (@w) {
                                             #path
                                             if ($w=~/^path *=/) {
                                                  $w=~s/^.*= *//;
                                                  $w=~s/^\///;                        #ex tank/media_shareafp
                                                  $zfs=$w;

                                                  $t=$w."_shareafp";
                                                  $zfs{$t}="on";
                                                  last;
                                             }
                                    }

                                    # get other values
                                    foreach $w (@w) {
                                            # time machine = yes
                                             if ($w=~/^time machine *=/) {
                                                  $w=~s/^.*= *//;
                                                  $t=$zfs."_afp_time_machine";
                                                  $zfs{$t}="$w";
                                             }


                                             # directory perm = 0770
                                             if ($w=~/^directory perm *=/) {
                                                  $w=~s/^.*= *//;
                                                  $t=$zfs."_afp_directory_perm";
                                                  $zfs{$t}="$w";
                                             }

                                             # file perm = 0660
                                             if ($w=~/^file perm *=/) {
                                                  $w=~s/^.*= *//;
                                                  $t=$zfs."_afp_file_perm";
                                                  $zfs{$t}="$w";
                                             }
                                   }


                  }

                  #debug
            #      &show_hash('zfs');
           #       &print_array(@w);


          } else { # netatalk 2
              if (-f "/etc/netatalk/AppleVolumes.default") {
                  $ant=&exe("cat /etc/netatalk/AppleVolumes.default",'','get zfs 441');

                  # get shares (lines beginning with /) ex.                               # /tank/timemachine TimeMachine allow:timemach options:tm
                                                                                          # /tank/media media forceuid:media
                                                                                          # /tank/software software forceuid:software
                  @l=();
                  @l=split(/\n/,$ant);
                  foreach $l (@l) {
                          if (!($l=~/^\//)) { next; }
                          @w=();
                          $l=~s/ +/ /g;
                          @w=split(/ /,$l);
                          $t=$w[0]."_shareafp";
                          $t=~s/^\///;                        #ex tank/media_shareafp
                          $zfs{$t}=$w[1];

                          #opt1
                          $t=$w[0]."_shareafp_opt";
                          $t=~s/^\///;                        #ex allow:timemach options:tm

                          $zfs{$t}="$w[2] $w[3] $w[4]";
                          $zfs{$t}=~s/ +$//;
                  }
              }
          }


   # write config: now acc
   # $p="/tmp/nappit/zfs.cfg";
   # &exe("chmod 666 $p");

   # $zfs{'buffertime'}=time();
   # &hash2file('zfs',$p);


  #test
  #&show_hash('zfs');
  #&print_array(@w);


  }

} #/if 0



# eof
1;