#! sharelib.pl
# functions for smb and nfs shares
#
# (c) napp-it 2010-2024
# fill %zfs with 'zfs get all' and editable parameter with 'zfs set'
# basics from zfs list in %zpool, already set in getbasics-lib.pl (allpools, syspool, datapools)


######################## Solaris and Illumos
sub sharelib_enable_solsmb {
#########################
    if (!($current{'on'}=~/solaris/i)) {  &mess("Illumoa/Solaris only!");   }
    my ($pam,$r);

    # smb settings set by cs installer
    # other   password required   pam_smb_passwd.so.1 nowarn

    if ($current{'on'}=~/Oracle Solaris/i) {
        $pam="/etc/pam.d/other";
    }  else {     # Illumos
        $pam="/etc/pam.conf";
    }

    $r=&exe("cat $pam");

    if ($r=~/pam_smb_passwd.so.1/s) {
       print "<br>pam configuration ok: 'other   password required   pam_smb_passwd.so.1 nowarn'<br>";
       sleep(1);
    } else {     # # command: appendfile::file::data::EndOfFile::
       print "enble smb in pam.conf<br>";
       &exe("appendfile::$pam\::# smb settings set by cs web-gui<n>other        password required   pam_smb_passwd.so.1 nowarn::EndOfFile::");
    }

    # start smb/server
    $r=&exe("svcs grep | smb/server");
    if (!($r=~/online/s)) {
       print "enble smb/server<br>";
       &exe("svcadm enable smb/server");
    }

    #&mess("$r");
}

######################## Solaris and Illumos
sub sharelib_enable_solguest {
#########################
if (!($current{'on'}=~/solaris/i)) {  &mess("Illumoa/Solaris only!");   }

my ($r,@p,$uid);
#
#if ($current{'on'}=~/Oracle Solaris/i) { idmapping winguest: unix:nobody  should be enough
# or create a user guest without pw (Illumos) and enable

$r=&exe("cat /etc/passwd");                 # get user?

if (!($r=~/\nguest:/s))   {

   print "<br>add user guest<br>";
   #
   $uid=60000;                              # guest uid
   @t=split(/\n/,$r);
   foreach my $t (@t) {
     @p=split(/:/,$t);                      # nobody:x:60001:60001:NFS Anonymous Access User:/:
     if ($p[2] > $uid) { $uid=$p[2]+1; }
   }

   # guest user
   $t="guest:x:$uid\:$uid\:smb guest account:/export/home/guest:/bin/false";

   # append to passwd
   &exe("appendfile::/etc/passwd::$t\::EndOfFile::");
}

# smbuser guest in /var/smb/smbpasswd     # guest:101::
$r=&exe("cat /var/smb/smbpasswd");        # user?

if (!($r=~/\bguest:/s))   {
   # append to smbpasswd

   #get next uid from /etc/passwd
   $r=&exe("cat /etc/passwd");               #

   $uid=60000;                              # guest uid
   @t=split(/\n/,$r);
   foreach my $t (@t) {
        @p=split(/:/,$t);                   # nobody:x:60001:60001:NFS Anonymous Access User:/:
        if ($p[0] eq "guest")  {
            $uid=$p[2];
            last;
        }
        if ($p[2] > $uid) { $uid=$p[2]+1; }
   }

  $t="guest:$uid\::";
  &exe("appendfile::/var/smb/smbpasswd::$t\::EndOfFile::");
}

# enable user guest
print "<br>enable user guest<br>";
&exe("smbadm enable-user guest");

#debug
#$r=&exe("cat /var/smb/smbpasswd");
#&mess($r);

#&mess('todo guest');

}

##################
sub sharelib_get_defaults {
################

    ##samba defaults
    #################
    # defaults for new share
    $current{'samba_defaults'}=<<EoF;
# valid users = \@ADgroup
guest ok = yes
read only = no
EoF

    # minimal global
    $current{'samba_minimal'}=<<EoF;
[global]
unix charset = UTF-8
workgroup = WORKGROUP
server string = FreeBSD
guest account = nobody
create mask = 0775
directory mask = 0755
browsable = yes
map to guest = bad user
EoF

$current{'pool_mount'}="";                     # mount pool under
$current{'samba_conf'}="-";

    ###############################
    if ($current{'on'}=~/bsd/i) {               #  bsd
    ###############################
        # /usr/local/etc/smb4.conf
        $current{'samba_conf'}=&exe("sc");       # samba config
        if (!($current{'samba_conf'}=~/smb/)) {
              $current{'samba_conf'}="/usr/local/etc/smb4.conf";         # default free-bsd 14
        }
        $current{'samba_restart'}="service samba_server restart";
    }

    ###############################
    if ($current{'on'}=~/linux/i) {               #  Proxmox, debian  default /etc/samba/smb.conf
    ###############################
         $current{'samba_conf'}=&exe("sc");       # samba config
         $current{'samba_restart'}="systemctl restart samba";

         if ($current{'on'}=~/truenas/i) {
             $current{'samba_restart'}="midclt call service.restart cifs";
             $current{'pool_mount'}="/mnt";
         }
    }

    ###################################
    if ($current{'on'}=~/solaris/i) {               #  Proxmox, debian  default /etc/samba/smb.conf
    ###################################
         $current{'samba_conf'}=&exe("sc");         # samba config or smb/server
         $current{'samba_restart'}="";              # samba only

    }

}  # /sub get defaults



#############################
sub sharelib_get_smbshares  {
#############################
    my $par=$_[0];                                   # 1 -> return samba.conf
    my ($l,@l,$r,$p,@p,@s,$k,$s);

    &sharelib_get_defaults;

    ################################                  #
    if ($current{'on'}=~/linux|bsd/i) {               #  SAMBA on Proxmox, debian, bsd
    ################################
         # smb(4).conf

         # [data]
         #    path = /tank/data                       # is filesystem?
         #    #valid users = @ADgroup
         #    guest ok = yes
         #    read only = no
         #    write permission = yes

         # console command: 'testparm'  checks smb.conf for syntax errors
         if ($current{'samba_conf'} ne "") {
            $r=&exe("cat $current{'samba_conf'}");
         }


         # if smb.conf is missing ex on free-bsd after samba install
         if ($r=~/No such file/i)   {

              $r="";
              ###################
              # write new smb.conf
              ####################

              # command: writefile::file::data::EndOfFile::
              $t="writefile::$current{'samba_conf'}\::$current{'samba_minimal'}\::EndOfFile::";                      # writefile default
              $t=~s/\n/<n>/gs;         # single line
              $r=&exe("$t");
              if (!($r=~/ok/)) {  &mess("create default smb.conf<hr>$r");  }
         }

         $r=~s/\r//gs;   # unwanted
         if (!($r=~/\[global/s))  { $r="\n$r"; }   # at least a newline to detect shares afterwrds with \n[share]

         @s=split(/\n\[/,$r);       # \n[.. areas

         # [0] -> '#\n# Sample configuration file for the Samba suite for Debian GNU/Linux.\n#\n#\n# This is..
         # [1] -> 'global]\n\nforce create mode = 0644\nforce directory mode = 0744\n\n## Browsing/Identification ###\n\n# Change this to the workgroup/NT-domain name your Samba server will part of\n   workgroup = WORKGROUP\n\n# server string is the equivalent of the NT Description field\n   server string = %h server (Samba, Ubuntu)\n\n# Windows Internet Name Serving Support Section:\n# WINS Support - Tells the NMBD component of Samba to enable its WINS Server\n#   wins support = no\n\n# WINS Server - Tells the NMBD components of Samba to be a WINS Client\n# Note: Samba can be either a WINS Server, or a WINS Client, but NOT both\n;   wins server = w.x.y.z\n\n# This will prevent nmbd to search for NetBIOS names through DNS.\n   dns proxy = no\n\n# What naming service and in what order should we use to resolve host names\n# to IP addresses\n;   name resolve order = lmhosts host wins bcast\n\n#### Networking ####\n\n# The specific set of interfaces / networks to bind to\n# This can be either the interface name or an IP address/netmask;\n# interface names are normally preferred\n;   interfaces = 127.0.0.0/8 eth0\n\n# Only bind to the named interfaces and/or networks; you must use the\n# 'interfaces' option above to use this.\n# It is recommended that you enable this feature if your Samba machine is\n# not protected by a firewall or is a firewall itself.  However, this\n# option cannot handle dynamic or non-broadcast interfaces correctly.\n;   bind interfaces only = yes\n\n\n\n#### Debugging/Accounting ####\n\n# This tells Samba to use a separate log file for each machine\n# that connects\n   log file = /var/log/samba/log.%m\n\n# Cap the size of the individual log files (in KiB).\n   max log size = 1000\n\n# If you want Samba to only log through syslog then set the following\n# parameter to 'yes'.\n#   syslog only = no\n\n# We want Samba to log a minimum amount of information to syslog. Everything\n# should go to /var/log/samba/log.{smbd,nmbd} instead. If you want to log\n# through syslog you should set the following parameter to something higher.\n   syslog = 0\n\n# Do something sensible when Samba crashes: mail the admin a backtrace\n   panic action = /usr/share/samba/panic-action %d\n\n\n####### Authentication #######\n\n# "security = user" is always a good idea. This will require a Unix account\n# in this server for every user accessing the server. See\n# /usr/share/doc/samba-doc/htmldocs/Samba3-HOWTO/ServerType.html\n# in the samba-doc package for details.\n#   security = user\n\n# You may wish to use password encryption.  See the section on\n# 'encrypt passwords' in the smb.conf(5) manpage before enabling.\n   encrypt passwords = true\n\n# If you are using encrypted passwords, Samba will need to know what\n# password database type you are using.\n   passdb backend = tdbsam\n\n   obey pam restrictions = yes\n\n# This boolean parameter controls whether Samba attempts to sync the Unix\n# password with the SMB password when the encrypted SMB password in the\n# passdb is changed.\n   unix password sync = yes\n\n# For Unix password sync to work on a Debian GNU/Linux system, the following\n# parameters must be set (thanks to Ian Kahan <<kahan@informatik.tu-muenchen.de> for\n# sending the correct chat script for the passwd program in Debian Sarge).\n   passwd program = /usr/bin/passwd %u\n   passwd chat = *Enter\snew\s*\spassword:* %n\n *Retype\snew\s*\spassword:* %n\n *password\supdated\ssuccessfully* .\n\n# This boolean controls whether PAM will be used for password changes\n# when requested by an SMB client instead of the program listed in\n# 'passwd program'. The default is 'no'.\n   pam password change = yes\n\n# This option controls how unsuccessful authentication attempts are mapped\n# to anonymous connections\nmap to guest = bad user\n\n########## Domains ###########\n\n# Is this machine able to authenticate users. Both PDC and BDC\n# must have this setting enabled. If you are the BDC you must\n# change the 'domain master' setting to no\n#\n;   domain logons = yes\n#\n# The following setting only takes effect if 'domain logons' is set\n# It specifies the location of the user's profile directory\n# from the client point of view)\n# The following required a [profiles] share to be setup on the\n# samba server (see below)\n;   logon path = \\%N\profiles\%U\n# Another common choice is storing the profile in the user's home directory\n# (this is Samba's default)\n#   logon path = \\%N\%U\profile\n\n# The following setting only takes effect if 'domain logons' is set\n# It specifies the location of a user's home directory (from the client\n# point of view)\n;   logon drive = H:\n#   logon home = \\%N\%U\n\n# The following setting only takes effect if 'domain logons' is set\n# It specifies the script to run during logon. The script must be stored\n# in the [netlogon] share\n# NOTE: Must be store in 'DOS' file format convention\n;   logon script = logon.cmd\n\n# This allows Unix users to be created on the domain controller via the SAMR\n# RPC pipe.  The example command creates a user account with a disabled Unix\n# password; please adapt to your needs\n; add user script = /usr/sbin/adduser --quiet --disabled-password --gecos "" %u\n\n########## Printing ##########\n\n# If you want to automatically load your printer list rather\n# than setting them up individually then you'll need this\n#   load printers = yes\n\n# lpr(ng) printing. You may wish to override the location of the\n# printcap file\n;   printing = bsd\n;   printcap name = /etc/printcap\n\n# CUPS printing.  See also the cupsaddsmb(8) manpage in the\n# cupsys-client package.\n;   printing = cups\n;   printcap name = cups\n\n############ Misc ############\n\n# Using the following line enables you to customise your configuration\n# on a per machine basis. The %m gets replaced with the netbios name\n# of the machine that is connecting\n;   include = /home/samba/etc/smb.conf.%m\n\n# Most people will find that this option gives better performance.\n# See smb.conf(5) and /usr/share/doc/samba-doc/htmldocs/Samba3-HOWTO/speed.html\n# for details\n# You may want to add the following on a Linux system:\n#         SO_RCVBUF=8192 SO_SNDBUF=8192\n#   socket options = TCP_NODELAY\n\n# The following parameter is useful only if you have the linpopup package\n# installed. The samba maintainer and the linpopup maintainer are\n# working to ease installation and configuration of linpopup and samba.\n;   message command = /bin/sh -c '/usr/bin/linpopup "%f" "%m" %s; rm %s' &\n\n# Domain Master specifies Samba to be the Domain Master Browser. If this\n# machine will be configured as a BDC (a secondary logon server), you\n# must set this to 'no'; otherwise, the default behavior is recommended.\n#   domain master = auto\n\n# Some defaults for winbind (make sure you're not using the ranges\n# for something else.)\n;   idmap uid = 10000-20000\n;   idmap gid = 10000-20000\n;   template shell = /bin/bash\n\n# The following was the default behaviour in sarge,\n# but samba upstream reverted the default because it might induce\n# performance issues in large organizations.\n# See Debian bug #368251 for some of the consequences of *not*\n# having this setting and smb.conf(5) for details.\n;   winbind enum groups = yes\n;   winbind enum users = yes\n\n# Setup usershare options to enable non-root users to share folders\n# with the net usershare command.\n\n# Maximum number of usershare. 0 (default) means that usershare is disabled.\n;   usershare max shares = 100\n\n# Allow users who've been granted usershare privileges to create\n# public shares, not just authenticated ones\n   usershare allow guests = yes\n\n#======================= Share Definitions =======================\n\n# Un-comment the following (and tweak the other settings below to suit)\n# to enable the default home directory shares.  This will share each\n# user's home directory as \\server\username\n;[homes]\n;   comment = Home Directories\n;   browseable = no\n\n# By default, the home directories are exported read-only. Change the\n# next parameter to 'no' if you want to be able to write to them.\n;   read only = yes\n\n# File creation mask is set to 0700 for security reasons. If you want to\n# create files with group=rw permissions, set next parameter to 0775.\n;  create mask = 0644\n\n# Directory creation mask is set to 0700 for security reasons. If you want to\n# create dirs. with group=rw permissions, set next parameter to 0775.\n;  directory mask = 0775\n\n# By default, \\server\username shares can be connected to by anyone\n# with access to the samba server.  Un-comment the following parameter\n# to make sure that only "username" can connect to \\server\username\n# This might need tweaking when using external authentication schemes\n;   valid users = %S\n\n# Un-comment the following and create the netlogon directory for Domain Logons\n# (you need to configure Samba to act as a domain controller too.)\n;[netlogon]\n;   comment = Network Logon Service\n;   path = /home/samba/netlogon\n;   guest ok = yes\n;   read only = yes\n;   share modes = no\n\n# Un-comment the following and create the profiles directory to store\n# users profiles (see the "logon path" option above)\n# (you need to configure Samba to act as a domain controller too.)\n# The path below should be writable by all users so that their\n# profile directory may be created the first time they log on\n;[profiles]\n;   comment = Users profiles\n;   path = /home/samba/profiles\n;   guest ok = no\n;   browseable = no\n;   create mask = 0600\n;   directory mask = 0700\n'
         # [2] -> 'printers]\n   comment = All Printers\n   browseable = no\n   path = /var/spool/samba\n   printable = yes\n   guest ok = no\n   read only = yes\n   create mask = 0700\n'
         # [3] -> 'data2]\n   path = /tank/data2\n   # valid users = @ADgroup\n   guest ok = yes\n   read only = no\n'
         # [4] -> 'osx]\n   path = /tank/osx\n   # valid users = @ADgroup\n   guest ok = yes\n   read only = no\n'
         # [5] -> 'data3]\n   path = /tank/data3\n   # valid users = @ADgroup\n   guest ok = yes\n   read only = no\n'

        if (@s==0)  { return; }           # no zfs shares

# &print_array(@s);

        my @fs=split(/\t/,$zfs{'allfs'});  # filesystems
        if (@fs==0)  { return; }           # no zfs fs

        foreach my $s (@s) {               # shares
               $s=~s/\r//gs;               # unwanted \r

               if ($s=~/^global|^printers/)     { next; }    # global/printer def
               if (!($s=~/path *=/s))           { next; }    # path = is needed for a share

               @p=split(/\n/,$s);

               $p[0]=~s/\]//;        # sharename     data]  -> data

               # [0] -> 'data2]'
               # [1] -> '   path = /tank/data2'
               # [2] -> '   # valid users = @ADgroup'
               # [3] -> '   guest ok = yes'
               # [4] -> '   read only = no'

               foreach my $p (@p) {                          # look for path

                    if (!($p=~/path *= *\//))    { next; }   # not path = line

                    $p=~s/.*path *= *//;                     # keep path ex /tank/data

                    # is $p a filesystem?
                    foreach my $f (@fs) {
                       if ($p eq "/$f") {                         # hit
                           $k="$f\_mysharesmb";
                           $zfs{$k}=$p[0];                        # zfs(tank/data) -> data
                       }
                    }

               } # foreach paths

        } #/foreach shares

        if ($par) { return($r); }                             # return smb.conf

    } #/bsd/linux

    ##############################                            # get solaris shares
    if ($current{'on'}=~/solaris/i) {                         # kernelased or samba
    ##############################

        # kernelbased
        if  ($current{samba_conf} =~/server/)  {               #smb/server

               if ($current{'on'}=~/Oracle Solaris/i) {
                  $r=&exe("zfs get -H -o value share | grep prot=smb");

                  #name=backup,path=/tank/backup,prot=smb
                  #name=tank_backup_enc,path=/tank/backup/enc,prot=smb
                  #name=test,path=/tank/backup/test,prot=smb

 #&mess($r);
                  my @z=split(/\n/,$r);
                  foreach my $z (@z) {                             # shares
                            @t=split(/,/,$z);                      # share prop

                            $s="";
                            $k="";

                            foreach my $t (@t) {
                              if ($t=~/path=/)        {  $k=$t; $k=~s/path=\///; }
                              if ($t=~/name=/)        {  $s.=",$t"; }
                              if ($t=~/guestok=true/) {  $s.=",$t"; }
                              if ($t=~/abe=true/)     {  $s.=",$t"; }
                            }
                            $s=~s/^,//;

                            if ($k ne "") {
                               $k.="_mysharesmb";
                               $zfs{$k}=$s;

                               $zfs{$k}=~s/name=//;
                               $zfs{$k}=~s/abe=true/a/;
                               $zfs{$k}=~s/guestok=true/g/;
                               $zfs{$kt}=~s/,+/,/;
                            }
                  }


               } else {  # Illumos

                  $r=&exe("zfs get -H -t filesystem sharesmb");

                  # tank/data     sharesmb     off     default
                  # tank/data2    sharesmb     name=data2     local

                  @l=split(/\n/,$r);
                  foreach my $l (@l) {
                       #
                       @p=split(/\t/,$l);

                       # [0] -> 'tank/data'
                       # [1] -> 'sharesmb'
                       # [2] -> 'off'               # or name=date,guestok=true,abe=true
                       # [3] -> 'default'

                       $t="$p[0]\_mysharesmb";
                       $zfs{$t}=$p[2];
                       $zfs{$t}=~s/name=//;
                       $zfs{$t}=~s/abe=true/a/;
                       $zfs{$t}=~s/guestok=true/g/;
                       $zfs{$t}=~s/,+/,/;

                       #&print_array(@p);
                  }

               }  #/sol/ill




               #&mess($r);
        }  #/kernel smb
    } # /solaris

    #&print_hash(%zfs);

    #############################                              # get windows shares
    if ($current{'on'}=~/mswin/i) {
    ##############################
        #
        my $r=&exe("get-smbshare");
        if ($r=~/error/) { &mess("powershell get-smbshare<hr>$r"); }

        # get-smbshare
        # Name   ScopeName Path       Description
        # ----   --------- ----       -----------
        # ADMIN$ *         C:\Windows Remoteverwaltung
        # C$     *         C:\        Standardfreigabe
        # IPC$   *                    Remote-IPC
        # data   *         x:\data    zfs:tank/data

        my @l=split(/\n/,$r);
        foreach my $l (@l) {

           if ($l=~/zfs:\w+/) {
               @p=split(/ +/,$l);
               $p[3]=~s/zfs://;
               $k="$p[3]\_mysharesmb";
               $zfs{$k}=$p[0];
           }
        }

    } # /windows

######################  /os ###############

} #/sub get shares


################################################      #  set smb_shares
sub sharelib_set_smbshare {
################################################

    #[sharename]
    #  path = /xx
    # ..

    if ($current{'on'}=~/Truenas/i) {  &mess('You must set Shares in the TrueNAS GUI'); }
    if ($current{'on'}=~/Darwin/i)  {  &mess('You must set Shares in OSX'); }

    ############################################
    # my $par=$_[0];  # opt par
    # command: writefile::file::data::EndOfFile::
    #############################################

    &sharelib_get_defaults;

    $in{'smb_settings'}=~s/\r//gs;
    $in{'smb_settings'}=~s/\n+$//gs;
    $in{'smb_settings'}=~s/\n */\n   /gs;

    $in{'text'}=~s/ +//gs;
    $in{'zfs'}=~s/ +//gs;

    if ($in{'text'} eq "")           {  &mess('sharename needed'); }
    if ($in{'prop'} ne "smb_share:") {  &mess('command needed'); }
    if ($in{'zfs'} eq "")            {  &mess('zfs fs needed'); }
    $t="$in{'zfs'}\_mountpoint";
    if (!($zfs{$t}=~/$in{'zfs'}\b/)) {  &mess('sharing only possible for mounted filesystems');  }
    if ($in{'member'} eq "")         {  &mess('member needed'); }
    if ($current{'samba_conf'}=~/\.conf/) {
       if ($in{'restart_cmd'} eq "")    {  &mess('restart cmd needed'); }
       if ($in{'smb_settings'} eq "")   {  &mess('share settings needed'); }
    }


    ##################################
    if ($current{'on'}=~/linux|bsd/i) {               #  Proxmox, debian, BSD
    #################################
         # smb.conf
         # [data]
         #    path = /tank/data
         #    #valid users = @Abteilungsgruppe
         #    guest ok = yes
         #    read only = no
         #    write permission = yes

         # read current config
         my $r=&exe("cat $current{'samba_conf'}");

         #if ($r=~/error/) { &mess("read smb.conf<hr>$r"); }
         $r=~s/\r//gs;   # unwanted

         # &mess($r);
         # already an entry
         if ($r=~/\n *path *= *\/$in{'zfs'}\n/s)  {
           &mess("Share entry for path = /$in{'zfs'} exists");
         }


         ####################
         ## share section ###
         ####################

my $share=<<EoF;                 # share setting
[$in{'text'}]
   path = $current{'pool_mount'}/$in{'zfs'}
   $in{'smb_settings'}

EoF
         ###################
         # write new smb.conf
         ####################

         # command: writefile::file::data::EndOfFile::
         $t="writefile::$current{'samba_conf'}\::$r\n$share\::EndOfFile::";   # writefile str
         $t=~s/\n/<n>/gs;         # single line
         $r=&exe("$t");
         if (!($r=~/ok/)) {  &mess("update smb.conf<hr>$r");  }

         # set permissions for shared folder  (owner and ownergroup)
         if ($in{'permissions'}=~/modify/) {                                    # everypne modify 777 for folder
               $r=&exe("chmod 777 $current{'pool_mount'}/$in{'zfs'}");          # pool_mount=mnt ex on truenas
         }

         if ($in{'permissions'}=~/read/) {                                      # everypne modify 775 for folder
              $r=&exe("chmod 775 $current{'pool_mount'}/$in{'zfs'}");
         }

         if ($in{'permissions'}=~/owner/) {                                     # everypne modify 770 for folder
              $r=&exe("chmod 770 $current{'pool_mount'}/$in{'zfs'}");
         }
         if ($r=~/error/) { &mess("chmod  $current{'pool_mount'}/$in{'zfs'}<hr>$r"); }

         ###############
         # restart samba
         ###############
         $r= &exe("$in{'restart_cmd'}");
         if ($r=~/error/) { &mess("restart Samba <hr>$r"); }


    } #/bsd, linux (samba)

    ###############################
    if ($current{'on'}=~/mswin/i) {               #  windows
    ##############################
          #
            #name of "S-1-1-0 -> everyone/jeder etc
          #Get-LocalGroup | Select-Object Name, SID

          $r=&exe("ps LocalGroup | Select-Object Name, SID");
          if ($r=~/error/) { &mess("powershell Localgroup<hr>$r"); }

          #Name                                        SID
          #----                                        ---
          #Administratoren                             S-1-5-32-544
          #Hauptbenutzer                               S-1-5-32-547
          #Benutzer                                    S-1-5-32-545

          @t=split(/\n/,$r);
          foreach my $t (@t) {
            #print "$t<br>";
            $t=~s/ +$//;
            if ($t=~/545$/) {                           # user group
              $t=~s/\s+S\-.*//;
              $ug=$t;
              next;
            }
            if ($t=~/544$/) {                           # admin group
              $t=~s/ +S\-.*//;
              $ag=$t;
            }
          }
          #&mess("$ug,$ag<hr>$t");
          #print "allow $ag = full<br>";;

          # driveletter
          $fl=&exe("zfs get -H -o value driveletter $in{'zfs'}");     # filesystem level
          if ($fl=~/error/) { &mess("zfs get<hr>$fl"); }

          chomp($fl);
          $t=$in{'zfs'};     # tank/data
          $t=~s/\/.*//;      # tank
          $pl=&exe("zfs get -H -o value driveletter $t");             # poollevel
          if ($pl=~/error/) { &mess("zfs get<hr>$pl"); }

          #chomp($pl);
          #&print_var($dl);

          # driveletter?
          if (($pl eq "off") || ($fl eq "off")){
            &mess("With driveletter='off', you must enable sharing in computermanagement or explorer");
          }
          if ($pl eq "-") {
            &mess("Set a driveletter for your pool or you must enable sharing in computermanagement or explorer");
          }

          #&print_var("$pl\n$fl");
          $i=$in{'zfs'};  # tank/data
          if ( $fl eq "-") {
                $fl=$pl;                 # fs drivelatter=pool and path is $pl:\filesystem
          }

          $fs=$in{'zfs'};
          $fs=~s/.*\///;                   #name of filesystem
          $i=$fs;


          $fl=~s/[^a-z]//gs;

          $i="$fl:\\$i";
          $i=~s/\//\\/g;           # ex g:\tank\data
          #&mess ($i);
          # New-SmbShare -Name data -path "f:\tank\data" -Description "tank/data" -fullAccess Administratoren -changeaccess Hauptbenutzer -CachingMode "None""

          $t="New-SmbShare -Name $in{'text'} -path \"$i\" -Description \"zfs:$in{'zfs'}\" -fullAccess \"$ag\" -changeaccess \"$ug\" -CachingMode \"None\"";
          print "create share $i (everyone=modify, admins=full, you can modify in computermanagement)<hr>";
          print "$t<br>";
          print &exe($t);

    }

}  # /sub set share

################################################  unset smb_shares
sub sharelib_unset_smbshare {
################################################

    if ($current{'on'}=~/Truenas/i) {  &mess('You must set Shares in the TrueNAS GUI'); }
    if ($current{'on'}=~/Darwin/i)  {  &mess('You must set Shares in OSX'); }

    &sharelib_get_defaults;

    # my $par=$_[0];  # opt par
    #
    ##################################
    if ($current{'on'}=~/linux|bsd/i) {               #  SAMBA on Proxmox, debian, bsd
    #################################
         my $r=&sharelib_get_smbshares(1);            # smb.conf

         # unshare: $in(zfs)

         $r=~s/\r//gs;   # unwanted
         @s=split(/\n\[/,$r);       # \n[areas ..]

         # [0] -> '#\n# Sample configuration file for the Samba suite for Debian GNU/Linux.\n#\n#\n# This is..
         # [1] -> 'global]\n\nforce create mode = 0644\nforce directory mode = 0744\n\n## Browsing/Identification ###\n\n# Change this to the workgroup/NT-domain name your Samba server will part of\n   workgroup = WORKGROUP\n\n# server string is the equivalent of the NT Description field\n   server string = %h server (Samba, Ubuntu)\n\n# Windows Internet Name Serving Support Section:\n# WINS Support - Tells the NMBD component of Samba to enable its WINS Server\n#   wins support = no\n\n# WINS Server - Tells the NMBD components of Samba to be a WINS Client\n# Note: Samba can be either a WINS Server, or a WINS Client, but NOT both\n;   wins server = w.x.y.z\n\n# This will prevent nmbd to search for NetBIOS names through DNS.\n   dns proxy = no\n\n# What naming service and in what order should we use to resolve host names\n# to IP addresses\n;   name resolve order = lmhosts host wins bcast\n\n#### Networking ####\n\n# The specific set of interfaces / networks to bind to\n# This can be either the interface name or an IP address/netmask;\n# interface names are normally preferred\n;   interfaces = 127.0.0.0/8 eth0\n\n# Only bind to the named interfaces and/or networks; you must use the\n# 'interfaces' option above to use this.\n# It is recommended that you enable this feature if your Samba machine is\n# not protected by a firewall or is a firewall itself.  However, this\n# option cannot handle dynamic or non-broadcast interfaces correctly.\n;   bind interfaces only = yes\n\n\n\n#### Debugging/Accounting ####\n\n# This tells Samba to use a separate log file for each machine\n# that connects\n   log file = /var/log/samba/log.%m\n\n# Cap the size of the individual log files (in KiB).\n   max log size = 1000\n\n# If you want Samba to only log through syslog then set the following\n# parameter to 'yes'.\n#   syslog only = no\n\n# We want Samba to log a minimum amount of information to syslog. Everything\n# should go to /var/log/samba/log.{smbd,nmbd} instead. If you want to log\n# through syslog you should set the following parameter to something higher.\n   syslog = 0\n\n# Do something sensible when Samba crashes: mail the admin a backtrace\n   panic action = /usr/share/samba/panic-action %d\n\n\n####### Authentication #######\n\n# "security = user" is always a good idea. This will require a Unix account\n# in this server for every user accessing the server. See\n# /usr/share/doc/samba-doc/htmldocs/Samba3-HOWTO/ServerType.html\n# in the samba-doc package for details.\n#   security = user\n\n# You may wish to use password encryption.  See the section on\n# 'encrypt passwords' in the smb.conf(5) manpage before enabling.\n   encrypt passwords = true\n\n# If you are using encrypted passwords, Samba will need to know what\n# password database type you are using.\n   passdb backend = tdbsam\n\n   obey pam restrictions = yes\n\n# This boolean parameter controls whether Samba attempts to sync the Unix\n# password with the SMB password when the encrypted SMB password in the\n# passdb is changed.\n   unix password sync = yes\n\n# For Unix password sync to work on a Debian GNU/Linux system, the following\n# parameters must be set (thanks to Ian Kahan <<kahan@informatik.tu-muenchen.de> for\n# sending the correct chat script for the passwd program in Debian Sarge).\n   passwd program = /usr/bin/passwd %u\n   passwd chat = *Enter\snew\s*\spassword:* %n\n *Retype\snew\s*\spassword:* %n\n *password\supdated\ssuccessfully* .\n\n# This boolean controls whether PAM will be used for password changes\n# when requested by an SMB client instead of the program listed in\n# 'passwd program'. The default is 'no'.\n   pam password change = yes\n\n# This option controls how unsuccessful authentication attempts are mapped\n# to anonymous connections\nmap to guest = bad user\n\n########## Domains ###########\n\n# Is this machine able to authenticate users. Both PDC and BDC\n# must have this setting enabled. If you are the BDC you must\n# change the 'domain master' setting to no\n#\n;   domain logons = yes\n#\n# The following setting only takes effect if 'domain logons' is set\n# It specifies the location of the user's profile directory\n# from the client point of view)\n# The following required a [profiles] share to be setup on the\n# samba server (see below)\n;   logon path = \\%N\profiles\%U\n# Another common choice is storing the profile in the user's home directory\n# (this is Samba's default)\n#   logon path = \\%N\%U\profile\n\n# The following setting only takes effect if 'domain logons' is set\n# It specifies the location of a user's home directory (from the client\n# point of view)\n;   logon drive = H:\n#   logon home = \\%N\%U\n\n# The following setting only takes effect if 'domain logons' is set\n# It specifies the script to run during logon. The script must be stored\n# in the [netlogon] share\n# NOTE: Must be store in 'DOS' file format convention\n;   logon script = logon.cmd\n\n# This allows Unix users to be created on the domain controller via the SAMR\n# RPC pipe.  The example command creates a user account with a disabled Unix\n# password; please adapt to your needs\n; add user script = /usr/sbin/adduser --quiet --disabled-password --gecos "" %u\n\n########## Printing ##########\n\n# If you want to automatically load your printer list rather\n# than setting them up individually then you'll need this\n#   load printers = yes\n\n# lpr(ng) printing. You may wish to override the location of the\n# printcap file\n;   printing = bsd\n;   printcap name = /etc/printcap\n\n# CUPS printing.  See also the cupsaddsmb(8) manpage in the\n# cupsys-client package.\n;   printing = cups\n;   printcap name = cups\n\n############ Misc ############\n\n# Using the following line enables you to customise your configuration\n# on a per machine basis. The %m gets replaced with the netbios name\n# of the machine that is connecting\n;   include = /home/samba/etc/smb.conf.%m\n\n# Most people will find that this option gives better performance.\n# See smb.conf(5) and /usr/share/doc/samba-doc/htmldocs/Samba3-HOWTO/speed.html\n# for details\n# You may want to add the following on a Linux system:\n#         SO_RCVBUF=8192 SO_SNDBUF=8192\n#   socket options = TCP_NODELAY\n\n# The following parameter is useful only if you have the linpopup package\n# installed. The samba maintainer and the linpopup maintainer are\n# working to ease installation and configuration of linpopup and samba.\n;   message command = /bin/sh -c '/usr/bin/linpopup "%f" "%m" %s; rm %s' &\n\n# Domain Master specifies Samba to be the Domain Master Browser. If this\n# machine will be configured as a BDC (a secondary logon server), you\n# must set this to 'no'; otherwise, the default behavior is recommended.\n#   domain master = auto\n\n# Some defaults for winbind (make sure you're not using the ranges\n# for something else.)\n;   idmap uid = 10000-20000\n;   idmap gid = 10000-20000\n;   template shell = /bin/bash\n\n# The following was the default behaviour in sarge,\n# but samba upstream reverted the default because it might induce\n# performance issues in large organizations.\n# See Debian bug #368251 for some of the consequences of *not*\n# having this setting and smb.conf(5) for details.\n;   winbind enum groups = yes\n;   winbind enum users = yes\n\n# Setup usershare options to enable non-root users to share folders\n# with the net usershare command.\n\n# Maximum number of usershare. 0 (default) means that usershare is disabled.\n;   usershare max shares = 100\n\n# Allow users who've been granted usershare privileges to create\n# public shares, not just authenticated ones\n   usershare allow guests = yes\n\n#======================= Share Definitions =======================\n\n# Un-comment the following (and tweak the other settings below to suit)\n# to enable the default home directory shares.  This will share each\n# user's home directory as \\server\username\n;[homes]\n;   comment = Home Directories\n;   browseable = no\n\n# By default, the home directories are exported read-only. Change the\n# next parameter to 'no' if you want to be able to write to them.\n;   read only = yes\n\n# File creation mask is set to 0700 for security reasons. If you want to\n# create files with group=rw permissions, set next parameter to 0775.\n;  create mask = 0644\n\n# Directory creation mask is set to 0700 for security reasons. If you want to\n# create dirs. with group=rw permissions, set next parameter to 0775.\n;  directory mask = 0775\n\n# By default, \\server\username shares can be connected to by anyone\n# with access to the samba server.  Un-comment the following parameter\n# to make sure that only "username" can connect to \\server\username\n# This might need tweaking when using external authentication schemes\n;   valid users = %S\n\n# Un-comment the following and create the netlogon directory for Domain Logons\n# (you need to configure Samba to act as a domain controller too.)\n;[netlogon]\n;   comment = Network Logon Service\n;   path = /home/samba/netlogon\n;   guest ok = yes\n;   read only = yes\n;   share modes = no\n\n# Un-comment the following and create the profiles directory to store\n# users profiles (see the "logon path" option above)\n# (you need to configure Samba to act as a domain controller too.)\n# The path below should be writable by all users so that their\n# profile directory may be created the first time they log on\n;[profiles]\n;   comment = Users profiles\n;   path = /home/samba/profiles\n;   guest ok = no\n;   browseable = no\n;   create mask = 0600\n;   directory mask = 0700\n'
         # [2] -> 'printers]\n   comment = All Printers\n   browseable = no\n   path = /var/spool/samba\n   printable = yes\n   guest ok = no\n   read only = yes\n   create mask = 0700\n'
         # [3] -> 'data2]\n   path = /tank/data2\n   # valid users = @ADgroup\n   guest ok = yes\n   read only = no\n'
         # [4] -> 'osx]\n   path = /tank/osx\n   # valid users = @ADgroup\n   guest ok = yes\n   read only = no\n'
         # [5] -> 'data3]\n   path = /tank/data3\n   # valid users = @ADgroup\n   guest ok = yes\n   read only = no\n'

         if (@s==0)  { return; }           # no zfs shares

         my $new;
         my $s;

#&print_hash(%in);
#&print_array(@s);

         my $i=0;
         foreach $s (@s) {

            if ($s ne "") {

               $i++;
               if ($i>1)   { $s="\[$s"; }

               if ($s=~/\n *path *= *\/$in{'zfs'}\n/s)  {
                 next;                       # remove this one
               } else {
                 $new.="$s\n";
               }
            }
         }

#&print_array(@s);
#&mess($new);                # new samba.conf

         ###################
         # write new smb.conf
         ####################

         # command: writefile::file::data::EndOfFile::
         $t="writefile::$current{'samba_conf'}\::$new\::EndOfFile::";           # writefile str
         $t=~s/\n/<n>/gs;         # single line

         $r=&exe("$t");
         if (!($r=~/ok/)) {  &mess("update smb.conf<hr>$r");  }

         ###############
         # restart samba
         ###############
         print &exe("$in{'restart_cmd'}");


         #&mess($new);
         #&print_array(@s);
         #&print_hash(%in);
         #&mess($r);

    }  #/linux/bsd

    ###############################
    if ($current{'on'}=~/mswin/i) {               # windows
    ###############################

       $r=&exe("Remove-SmbShare -Name $in{'prop2'} -force");
       if ($r=~/error/) { &mess("powershell get-smbshare<hr>$r"); }

    }

}  #/sub


########## do not modify below ##################
# eof
###
1;